%%%
%%% Q_LENSDISTORT: FIRST-ORDER RADIAL LENS DISTORTION
%%%
%%%		Distort image according to a one-parameter (k) radially
%%%		symmetric lens distortion:
%%%			x2 = x * (1 + k*(sqrt(x^2+y^2)))
%%%			y2 = y * (1 + k*(sqrt(x^2+y^2)))
%%%
%%% DATE: 	March 29, 2000
%%% BY:   	Hany Farid (farid@mit.edu), Copyright 1998, MIT.
%%%
function [] = Q_lensdistort( parent, first, qmode );

if( first )
	if( Q_active(parent,1) )
		return;
	end

	IM	= Q_get_iminfo( parent );
	Q_window( parent );
	Q_sliders( parent );
	Q_callbacks( parent );
	Q_lensdistort( str2num(sprintf('%d',parent)), 0, 1 );
else 
	if( qmode == 1 | qmode == 2 ) % PREVIEW
		vals	= Q_getvals( parent, qmode );	
		IM	= Q_get_iminfo( parent );
		im2	= lensdistort( Q_make_preview_im(IM.im), vals, '*linear' );
		Q_preview_im( im2, parent, 0 );
	elseif( qmode == 3 ) % OK
		vals	= Q_getvals( parent, qmode );	
		IM	= Q_get_iminfo( parent );
		IM.im	= lensdistort( IM.im, vals, '*cubic' );
		Q_resetfig( parent, IM );
		return;
	end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [] = Q_window( parent )

	Q_expandfig( parent, 'Lens Distortion', 1, 1 );

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [] = Q_sliders( parent )

	H 	= Q_get_handles( parent );

	%%% LENS DISTORTION SLIDER/EDIT/LABEL
	H.temp(1) = Q_input( parent, [0.6 0.37 0.25 0.06], ...
				[-1 1 0 0.01 0.02], 'slider' );
	H.temp(2) = Q_input( parent, [0.86 0.37 0.12 0.06], 0, 'edit' );
	H.temp(3) = Q_input( parent, [0.86 0.31 0.12 0.06], ...
				'K', 'text' );

	Q_set_handles( H, parent );

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [] = Q_callbacks( parent )

	H 	= Q_get_handles( parent );
	cmd1 	= sprintf( 'Q_lensdistort(%d,0,1);', parent );
	cmd2 	= sprintf( 'Q_lensdistort(%d,0,2);', parent );
	cmd3 	= sprintf( 'Q_lensdistort(%d,0,3);', parent );

	%%% SLIDER/EDIT
	set( H.temp(1), 'Callback', cmd1 );
	set( H.temp(2), 'Callback', cmd2 );
	set( H.ok, 'Callback', cmd3 );

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ vals ] = Q_getvals( parent, qmode )

	H	= Q_get_handles( parent );

	if( qmode == 1 | qmode == 3 | qmode == 4 ) % SLIDER INPUT
		K	= round( 100*get( H.temp(1), 'Value' ) )/100;
		set(  H.temp(2), 'String', K );
	elseif( qmode == 2 ) % EDIT INPUT
		K	= str2num( get( H.temp(2), 'String' ) );
		if( isempty(K) ) K = 0; end;
		K	= max( K, get(  H.temp(1), 'Min' ) );
		K	= min( K, get(  H.temp(1), 'Max' ) );
		set( H.temp(1), 'Value', K );
		set( H.temp(2), 'String', K );
	end

	vals 	= [K];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function[ im3 ] = lensdistort( im, vals, method )

	K = vals(1);
	if( K == 0 )
		im3 = im;
		return;
	end

	%%% DISTORT
	[ydim,xdim,zdim] = size( im );
	[xramp,yramp] = meshgrid( 2*([1:xdim]/xdim-0.5), 2*([1:ydim]/ydim-0.5) );
	r2	= sqrt( xramp.^2 + yramp.^2 );
	warpx	= xramp .* (1 + K*r2);
	warpy	= yramp .* (1 + K*r2);
	warpx 	= warpx / max(abs(warpx(:)));
	warpy 	= warpy / max(abs(warpy(:)));

	for z = 1 : zdim
		im2 = interp2( xramp, yramp, im(:,:,z), warpx, warpy, method );

		%%% REMOVE NANS DUE TO INTERP2
		im2	 = im2(:);
		index	 = find( isnan( im2 ));
		im2(index) = zeros( 1, size(index,1) );
		im3(:,:,z) = reshape( im2, ydim, xdim );
	end
