%%%
%%% Q_MATLABOPEN: READ MATLAB VARIABLE.
%%%
%%%		Load an existing matlab variable into a new Q figure.
%%%		Use default colormap/clim defined in Q_defaults.m 
%%%		
%%% DATE: 	June 3, 1998
%%% BY:   	Hany Farid (farid@mit.edu), Copyright 1998, MIT.
%%%
%%% MODIFY: 	March 29, 2000
%%%		Add support for color images.
%%%
function [] = Q_matlabopen( parent, first, qmode );

if( first )
	if( Q_active(parent,1) )
		return;
	end

	Q_window( parent );
	Q_sliders( parent );
	Q_callbacks( parent );
else
	if( qmode == 1 ) % PREVIEW 
		vals	= Q_getvals( parent, qmode );
		if( isempty( vals.name ) | ...
		    ~evalin('base',sprintf('exist(''%s'')==1',vals.name)) )
			Q_preview_im( 0, parent );
			fprintf( 'Matlab Open(): variable (%s) doesn''t exist\n',vals.name );
		else
			im = evalin( 'base', vals.name );
			im = Q_make_preview_im( im );
			IM = Q_get_iminfo( parent );
			[ydim,xdim,zdim] = size( im );
			if( zdim == 1 )
				cmap = Q_defaults( 'cmap', 'gray' );
				clim = Q_defaults( 'clim', 'gray' );
			end
			if( zdim == 3 )
				cmap = Q_defaults( 'cmap', 'rgb' );
				clim = Q_defaults( 'clim', 'rgb' );
			end
			Q_preview_im( im, parent, vals.auto, cmap, clim );
		end
	elseif( qmode == 2 ) % OK 
		vals	= Q_getvals( parent, qmode );
		if( isempty( vals.name ) | ...
		    ~evalin('base',sprintf('exist(''%s'')==1',vals.name)) )
			Q_preview_im( 0, parent );
			fprintf( 'Matlab Open(): variable (%s) doesn''t exist\n',vals.name );
		else
			im = evalin( 'base', vals.name );
			IM = Q_get_iminfo( parent );
			[ydim,xdim,zdim] = size( im );
			if( zdim == 1 )
				D = Q_defaults( 'all', 'gray' );
			end
			if( zdim == 3 )
				D = Q_defaults( 'all', 'rgb' );
			end
			Q_resetfig( parent, IM );
			if( vals.auto )
			   for z = 1 : zdim
			      D.clim(z,:)=[min(min(im(:,:,z))) max(max(im(:,:,z)))];
			   end
			end
			Q_main( im, D.cmap, D.clim, D.magnify, D.mode, ...
					vals.name );
		end
	end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [] = Q_window( parent )

	Q_expandfig( parent, 'Matlab Open', 1, 1 );

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [] = Q_sliders( parent )

	H		= Q_get_handles( parent );

	%%% FILENAME ENTRY
	H.temp(1)	= Q_input( parent, [0.6 0.38 0.35 0.1], ...
				'', 'edit' );
	set( H.temp(1), 'FontSize', 12 );
	set( H.temp(1), 'HorizontalAlignment', 'left' );

	%%% FILENAME LABEL
	H.temp(2) 	= Q_input( parent, [0.6 0.25 0.35 0.1], ...
				'Variable Name', 'text' );
	set( H.temp(2), 'FontSize', 12 );
	set( H.temp(2), 'HorizontalAlignment', 'left' );

	%%% AUTOSCALE OPTION
	H.temp(3) 	= Q_input( parent, [0.82 0.55 0.14 0.06], ...
				'Autoscale', 'checkbox' );
	Q_set_handles( H, parent );

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [] = Q_callbacks( parent )

	H	= Q_get_handles( parent );
	cmd1	= sprintf( 'Q_matlabopen( %d,0,1 );', parent );
	cmd2	= sprintf( 'Q_matlabopen( %d,0,2 );', parent );

	set( H.temp(1), 'Callback', cmd1 );
	set( H.temp(3), 'Callback', cmd1 );
	set( H.ok, 'Callback', cmd2 );

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ vals ] = Q_getvals( parent, qmode )

	H		= Q_get_handles( parent );	
	vals.name	= get( H.temp(1), 'String' ); % filename
	vals.auto	= get( H.temp(3), 'Value' ); % autoscale option
