%%%
%%% Q_SHOW_IM:	NON-MENU ROUTINE.
%%%
%%%		Display an image in figure(parent).  Set various
%%%		parameters to make things look nice.  Update menu
%%%		checkboxes for magnification and mode.  Update undo
%%%		buffer.  
%%%
%%% DATE: 	June 3, 1998
%%% BY:   	Hany Farid (farid@mit.edu), Copyright 1998, MIT.
%%%
%%% MODIFY:	March 29, 2000 
%%%		Add support for color images.  Matlab does not pass an
%%%		RGB image through the colormap - so we are forced to
%%%		do this ourselves.  Pass each color channel
%%%		individually through each of the three columns of the
%%%		colormap.  Clim now has three rows corresponding to
%%%		each color channel.
%%%
function[] = Q_show_im( IM, parent );

	%%% OPEN FIGURE AND ADJUST SIZE	
	figure( parent );
	axHandle = gca;
	set_position( IM, parent );
	H 	= Q_get_handles( parent );

	%%% PLACE OLD DATA INTO UNDO BUFFER
	IM_UNDO	= Q_get_iminfo( parent );

	%%% BUILD FILENAME/MAGNIFY/IMAGE SIZE
	[ydim,xdim,zdim]= size( IM.im );
	filename	= IM.filename( 1:min(10,size(IM.filename,2) ) );

	if( IM.magnify >= 1 )
		immag = sprintf( '[%dx] ', IM.magnify );
	elseif( IM.magnify == 0.5 )
		immag = sprintf( '[1/2x] ' );
	elseif( IM.magnify == 0.25 )
		immag = sprintf( '[1/4x] ' );
	elseif( IM.magnify == 0.125 )
		immag = sprintf( '[1/8x] ' );
	else
		immag = '';
	end
	imsize = sprintf( '[%dx%d]', ydim,xdim );

	set( H.immag, 'String', immag );
	set( H.imsize, 'String', imsize );
	set( parent, 'Name', sprintf('%d: %s', parent, IM.filename) );

	if( strcmp( IM.mode,'gray' ) )
		%%% BUILD DISPLAY AND IMAGE VALUE RANGE
		minim = min( IM.im(:) );
		maxim = max( IM.im(:) );
		if( abs(maxim)>=1 & abs(IM.clim(2))>=1 )
		   drng = sprintf('disp: [%.0f %.0f]',IM.clim(1), IM.clim(2) );
	   	   irng = sprintf('im: [%.0f %.0f]', minim, maxim );
		else
		   drng = sprintf('disp: [%.1f %.1f]',IM.clim(1), IM.clim(2) );
	   	   irng = sprintf('im: [%.1f %.1f]', minim, maxim );
		end
		set( H.drange, 'String', drng );
		set( H.irange, 'String', irng );

		%%% SHOW IMAGE
		figure( parent );
		imagesc( IM.im );
		axis image;
		set( axHandle, 'XTick', [], 'YTick', [] );
		set( axHandle, 'Units', 'normalized', 'Position', [0 0 1 1] );
		caxis( IM.clim );
		colormap( Q_clip_cmap(IM.cmap) ); 

		%%% UPDATE MENU CHECK (MODE)
		set( H.mode.gray, 'Checked', 'on' );
		set( H.mode.rgb, 'Checked', 'off' );
	end

	if( strcmp( IM.mode, 'rgb' ) )
		%%% BUILD DISPLAY AND IMAGE VALUE RANGE
		minimr 	= min(min(IM.im(:,:,1)));
		maximr 	= max(max(IM.im(:,:,1)));
		minimg 	= min(min(IM.im(:,:,2)));
		maximg 	= max(max(IM.im(:,:,2)));
		minimb 	= min(min(IM.im(:,:,3)));
		maximb 	= max(max(IM.im(:,:,3)));

		if( abs(maximr)>=1 & abs(IM.clim(1,2))>=1 )
		   drng = sprintf('disp: [%.0f %.0f] [%.0f %.0f] [%.0f %.0f]',...
			IM.clim(1,1), IM.clim(1,2), ...
			IM.clim(2,1), IM.clim(2,2), ...
			IM.clim(3,1), IM.clim(3,2) );
	   	   irng = sprintf('im: [%.0f %.0f] [%.0f %.0f] [%.0f %.0f]',...
			 minimr, maximr, minimg, maximg, minimb, maximb );
		else
		   drng = sprintf('disp: [%.1f %.1f] [%.1f %.1f] [%.1f %.1f]',...
			IM.clim(1,1), IM.clim(1,2), ...
			IM.clim(2,1), IM.clim(2,2), ...
			IM.clim(3,1), IM.clim(3,2) );
	   	   irng = sprintf('im: [%.1f %.1f] [%.1f %.1f] [%.1f %.1f]',...
			 minimr, maximr, minimg, maximg, minimb, maximb );
		end
		set( H.drange, 'String', drng );
		set( H.irange, 'String', irng );

		%%% SHOW IMAGE
                cmap    = Q_clip_cmap(IM.cmap); 
		im	= IM.im;
                [ydim,xdim,zdim] = size( im );
		for z = 1 : 3
			a	= IM.clim(z,1);
			b	= IM.clim(z,2);
			im2	= im(:,:,z);
			im2	= im2 - a;
			im2	= 255 * im2 / b;
			im2	= round( max( min( im2, 255 ), 0 ) );
			cmap2	= cmap(:,z);
			im2	= cmap2( im2(:)+1 );	% FAST LOOKUP
			im3(:,:,z) = reshape( im2, ydim, xdim ); 
		end
		figure( parent );
		imagesc( im3 );
		axis image;
	        set( axHandle, 'XTick', [], 'YTick', [] );
		set( axHandle, 'Units', 'normalized', 'Position', [0 0 1 1] );

		%%% UPDATE MENU CHECK (MODE)
		set( H.mode.gray, 'Checked', 'off' );
		set( H.mode.rgb, 'Checked', 'on' );
	end

	%%% UPDATE MENU CHECK (MAGNIFY)
	set( H.display.magnify1_8x, 'Checked', 'off' );
	set( H.display.magnify1_4x, 'Checked', 'off' );
	set( H.display.magnify1_2x, 'Checked', 'off' );
	set( H.display.magnify1x, 'Checked', 'off' );
	set( H.display.magnify2x, 'Checked', 'off' );
	set( H.display.magnify4x, 'Checked', 'off' );
	set( H.display.magnify8x, 'Checked', 'off' );

        if( IM.magnify == 0.125 )
                set( H.display.magnify1_8x, 'Checked', 'on' );
        elseif( IM.magnify == 0.25 )
                set( H.display.magnify1_4x, 'Checked', 'on' );
        elseif( IM.magnify == 0.5 )
                set( H.display.magnify1_2x, 'Checked', 'on' );
        else
                eval(sprintf('set(H.display.magnify%dx,''Checked'',''on'');',...
			 	IM.magnify ) );
        end

	%%% UPDATE CURRENT AND UNDO DATA
	h = get( parent, 'UserData' );

	h.image.im		= IM.im;
	h.image.cmap		= IM.cmap;
	h.image.clim		= IM.clim;
	h.image.filename	= IM.filename;
	h.image.magnify		= IM.magnify;
	h.image.mode		= IM.mode;

	h.image.undo.im		= IM_UNDO.im;
	h.image.undo.cmap	= IM_UNDO.cmap;
	h.image.undo.clim	= IM_UNDO.clim;
	h.image.undo.filename	= IM_UNDO.filename;
	h.image.undo.magnify	= IM_UNDO.magnify;
	h.image.undo.mode	= IM_UNDO.mode;

	set( parent, 'UserData', h );

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function[] = set_position( IM, parent );

	set( parent, 'Units', 'pixels' );
	pos = get( parent, 'Position' );
	if( strcmp( IM.mode, 'gray' ) )
		pos = [max( 20, pos(1) ) ...
		       max( 20, pos(2)-(IM.magnify*size(IM.im,1)-pos(4))-50 )...
		       IM.magnify * size(IM.im,2) ...
		       IM.magnify * size(IM.im,1) + 50];
	end
	if( strcmp( IM.mode, 'rgb' ) )
		pos = [max( 20, pos(1) ) ...
		       max( 20, pos(2)-(IM.magnify*size(IM.im,1)-pos(4))-75 )...
		       IM.magnify * size(IM.im,2) ...
		       IM.magnify * size(IM.im,1) + 75];
	end

	set( parent, 'Position', pos );
	set( parent, 'Units', 'normalized' );
